
--
-- Scratchpad of useful or interesting SQL snippets
-- 
-- $Id: queries.sql 972 2012-01-29 11:00:09Z matthew $
--

-- Which are the most popular makefiles included by ports?

SELECT 
    include.path, count(makefile_dependency.include_id) AS count
FROM
    makefile include INNER JOIN  makefile_dependency
        ON ( include.makefile_id = makefile_dependency.include_id )
GROUP BY
    include.path
ORDER BY
    count DESC,
    include.path ASC
;

-- Which are the most popular licenses used by ports?

SELECT
    license.license_name,
    count(license_dependency.port_id) AS count
FROM
    license NATURAL JOIN license_dependency
GROUP BY
    license.license_id
ORDER BY
    count DESC,
    license.license_name ASC
;

-- Ports overdue for expiration

SELECT
    ports_by_name.parent || '/' || ports_by_name.name as port,
    caveat::timestamp AS expired
FROM
    ports_by_name NATURAL JOIN caveat
WHERE
    caveat_type = 'EXPIRATION_DATE'
    AND caveat.caveat::timestamp < now()
ORDER BY
    expired
;

-- Ports that don't have a run-time dependency on anything (including
-- library dependencies)

SELECT
   ports_by_name.parent || '/' || ports_by_name.name AS port
FROM
   ports_by_name LEFT JOIN run_lib_depends USING (port_id)
WHERE
   ports_by_name.type = 'port'::port_object
   AND run_depends.dependency_id IS NULL
ORDER BY
   port
;

-- Ports that nothing depends upon at run-time, including library
-- dependencies

SELECT
   ports_by_name.parent || '/' || ports_by_name.name AS port
FROM
   run_lib_depends FULL JOIN ports_by_name
       ON (run_lib_depends.dependency_id = ports_by_name.port_id)
WHERE
   ports_by_name.type ='port'::port_object
   AND run_lib_depends.port_id IS NULL
ORDER BY
   port
;

-- Ports that neither have dependents nor that depend on
-- anything at run-time, including libraries

SELECT 
    ports.parent || '/' || ports.name AS port
FROM
    ports_by_name above
        FULL JOIN run_lib_depends a   ON ( above.port_id = a.port_id )
        FULL JOIN ports_by_name ports ON ( a.dependency_id = ports.port_id )
        FULL JOIN run_lib_depends b   ON ( ports.port_id = b.port_id )
        FULL JOIN ports_by_name below ON ( b.dependency_id = below.port_id )
WHERE
    ports.type = 'port'::port_object
    AND above.port_id IS NULL
    AND below.port_id IS NULL
ORDER BY
    port
;

-- The ports with the largest number of direct run-time dependencies
-- on other ports (including LIB_DEPENDS)

SELECT
   ports_by_name.parent || '/' || ports_by_name.name AS port,
   count(run_lib_depends.dependency_id) AS count
FROM
   ports_by_name NATURAL JOIN run_lib_depends
GROUP BY
   port
ORDER BY
   count DESC,
   port ASC
;

-- The ports that are directly depended on at build-time by the most
-- other ports (including LIB_DEPENDS)

SELECT
   ports_by_name.parent || '/' || ports_by_name.name AS port,
   count(build_lib_depends.dependency_id) AS count
FROM
   ports_by_name INNER JOIN build_lib_depends
     ON (ports_by_name.port_id = build_lib_depends.dependency_id)
GROUP BY
   port
ORDER BY 
   count DESC,
   port ASC
;

-- Which ports require use of which usernames?

SELECT
    ports_by_name.parent || '/' || ports_by_name.name as port,
    user_group.user_group_name
FROM
    ports_by_name 
        NATURAL JOIN user_group_dependency
        NATURAL JOIN user_group
WHERE
    user_group.user_group_type = 'user'::user_group_type
ORDER BY
    port
; 

-- What problems are there in the ports maintained by 'perl@FreeBSD.org'

SELECT
    ports_by_name.parent || '/' || ports_by_name.name AS port,
    caveat.caveat_type,
    caveat.caveat
FROM 
    ports_by_name
        NATURAL JOIN caveat
        NATURAL JOIN port
        NATURAL JOIN maintainer
WHERE
     maintainer.maintainer_email = 'perl@FreeBSD.org'
;

-- Which maintainer maintains the most ports?

SELECT
    maintainer.maintainer_email,
    count(port.port_id) AS count
FROM
    maintainer NATURAL JOIN port
GROUP BY
    maintainer.maintainer_email
ORDER BY
    count DESC,
    maintainer.maintainer_email ASC
;

-- What option names are most popular?

SELECT DISTINCT
    option_name, count(option_name)
FROM
    option
GROUP BY
    option_name
ORDER BY
    2 desc, 1
;

-- ports with non-unique LATEST_LINK values.  Since the latest_link
-- table excludes ports where NO_LATEST_LINK is set, anything produced
-- by this query is potentially a problem.  Except that setting make
-- variables like WITHOUT_X11=yes or PAPERSIZE=a4 will cause spurious
-- positives.

SELECT
    duplicates.latest_link,
    p.parent || '/' || p.name AS port_name,
    port.pkg_name
FROM
    ports_by_name p
        NATURAL JOIN port
        NATURAL JOIN latest_link 
        NATURAL JOIN (
            SELECT
                latest_link.latest_link
            FROM
                latest_link
            GROUP BY
                latest_link.latest_link
            HAVING
                count(latest_link.port_id) > 1
        ) AS duplicates
ORDER BY
    duplicates.latest_link,
    port_name
;

-- ports with both non-unique names (same name, but in different
-- categories) and non-unique latest-link settings.  Should be no
-- output -- any that does appear almost certainly indicates real
-- problems.

SELECT
    p.parent || '/' || p.name AS name,
    latest_link.latest_link,
    port.pkg_name
FROM
    ports_by_name p
       NATURAL JOIN port
       NATURAL JOIN latest_link
       NATURAL JOIN (
           SELECT
               pp.name,
               ll.latest_link
           FROM
               ports_by_name pp
                   NATURAL JOIN latest_link ll
           GROUP BY
               pp.name,
               ll.latest_link
           HAVING
               count(pp.name) > 1
               AND count(ll.latest_link) > 1
       ) AS dups 
ORDER BY
    p.name,
    p.parent
;

-- All the ports that use OPTIONS, and the OPTIONSFILE they refer to

SELECT
    p.parent || '/' || p.name as portname,
    o.optionsfile
FROM
    ports_by_name p 
        NATURAL JOIN optionsfile_dependency
        NATURAL JOIN optionsfile o;  

-- Ports that share an optionsfile with other ports

SELECT
    o.optionsfile,
    p.parent || '/' || p.name as portname
FROM
    ports_by_name p
        NATURAL JOIN optionsfile_dependency
        NATURAL JOIN optionsfile o
        NATURAL JOIN (
            SELECT
                optionsfile_id
            FROM
                optionsfile_dependency
            GROUP BY
                optionsfile_id
            HAVING
                count(port_id) > 1
        ) AS reused
    ORDER BY
    o.optionsfile,
    portname
;

-- Ditto, but formatted so that the list of ports is collapsed onto
-- one space separated line

SELECT
    o.optionsfile,
    string_agg( p.parent || '/' || p.name, ' ' ) as portname
FROM
    ports_by_name p
        NATURAL JOIN optionsfile_dependency
        NATURAL JOIN optionsfile o
        NATURAL JOIN (
            SELECT
                optionsfile_id
            FROM
                optionsfile_dependency
            GROUP BY
                optionsfile_id
            HAVING
                count(port_id) > 1
        ) AS reused
GROUP BY
    o.optionsfile
ORDER BY
    o.optionsfile
;

-- Package run-time dependencies, formatted similarly to columns in
-- the INDEX file. 

SELECT
    p.pkg_name,
    pn.parent || '/' || pn.name as portname,
    run.list AS "RUN Depends"
FROM
    ports_by_name pn NATURAL JOIN port p
        LEFT JOIN (
            SELECT
                de.port_id,
                string_agg(d.pkg_name, ' ' ORDER BY d.pkg_name) AS list
            FROM
                run_depends_accumulated de
                LEFT JOIN port d ON ( de.dependency_id = d.port_id )
            GROUP BY
                de.port_id
            ) AS run USING (port_id)
ORDER BY
    portname
;

-- Categories as listed in the INDEX.
SELECT
    p.parent || '/' || p.name AS portname, 
    c.list
FROM
    ports_by_name p
        NATURAL JOIN ( 
           SELECT
               cc.port_id,
               string_agg(cat.name, ' ' ORDER BY cc.ordering) AS list
           FROM
               category cc INNER JOIN ports_tree cat
                   ON (cc.category_id = cat.port_id)
           GROUP BY
               cc.port_id
        ) AS c
ORDER BY
    portname
;

-- ports containing distfiles with conflicting size and sha256 data:
-- this is almost certainly an error.

SELECT
    p.parent || '/' || p.name as port,
    d.distfile_path,
    d.distfile_size,
    d.distfile_sha256
FROM
    ports_by_name p NATURAL JOIN
    distfile_dependency NATURAL JOIN
    distfile d INNER JOIN (
    SELECT
            distfile.distfile_path
	    FROM
            distfile
	    GROUP BY
            distfile.distfile_path
	    HAVING
            count(distfile.distfile_id) > 1
       ) as dupes USING (distfile_path)
ORDER BY
    d.distfile_path
;

-- The biggie.  Generate an INDEX file like 'make index'.  Set
-- unaligned output mode (\a) and only show tuples (\t on) in psql for
-- the best result. Use \o to direct the output to a file.
--
-- Columns are:
--   PKGNAME
--   ORIGIN (use for sort-order)
--   PREFIX
--   COMMENT
--   DESCR
--   MAINTAINER
--   CATEGORIES
--   BUILD(+LIB) DEPENDS
--   RUN(+LIB) DEPENDS
--   WWW
--   EXTRACT DEPENDS
--   PATCH DEPENDS
--   FETCH DEPENDS

SELECT
    port.pkg_name,
    '/usr/ports/' || p.parent || '/' || p.name AS origin,
    port.prefix,
    port.comment,
    port.descr,
    maintainer.maintainer_email,
    cat.list,
    build.list,
    run.list,
    port.www,
    extract.list,
    patch.list,
    ffetch.list
FROM
    port
        NATURAL JOIN ports_by_name p
        NATURAL JOIN maintainer
        LEFT JOIN (
            SELECT
                cc.port_id,
                string_agg(ct.name, ' ' ORDER BY cc.ordering) AS list
            FROM
                category cc INNER JOIN ports_tree ct
                    ON (cc.category_id = ct.port_id)
            GROUP BY
                cc.port_id
            ) AS cat USING (port_id)
        LEFT JOIN (
            SELECT
                d.port_id,
                string_agg(p.pkg_name, ' ' ORDER BY p.pkg_name) AS list
            FROM
                run_depends_accumulated d INNER JOIN port p
                    ON ( d.dependency_id = p.port_id )
            GROUP BY
                d.port_id
            ) AS run USING (port_id)
        LEFT JOIN (
            SELECT
                d.port_id,
                string_agg(p.pkg_name, ' ' ORDER BY p.pkg_name) AS list
            FROM
                build_depends_accumulated d INNER JOIN port p
                    ON ( d.dependency_id = p.port_id )
            GROUP BY
                d.port_id
            ) AS build USING (port_id)
        LEFT JOIN (
            SELECT
                d.port_id,
                string_agg(p.pkg_name, ' ' ORDER BY p.pkg_name) AS list
            FROM
                extract_depends_accumulated d INNER JOIN port p
                    ON ( d.dependency_id = p.port_id )
            GROUP BY
                d.port_id
            ) AS extract USING (port_id)
        LEFT JOIN (
            SELECT
                d.port_id,
                string_agg(p.pkg_name, ' ' ORDER BY p.pkg_name) AS list
            FROM
                patch_depends_accumulated d INNER JOIN port p
                    ON ( d.dependency_id = p.port_id )
            GROUP BY
                d.port_id
            ) AS patch USING (port_id)
        LEFT JOIN (
            SELECT
                d.port_id,
                string_agg(p.pkg_name, ' ' ORDER BY p.pkg_name) AS list
            FROM
                fetch_depends_accumulated d INNER JOIN port p
                    ON ( d.dependency_id = p.port_id )
            GROUP BY
                d.port_id
            ) AS ffetch USING (port_id)
ORDER BY
    origin
;

--
-- That's All Folks!
--
