--
-- Create the portindex DB schema
--
-- $Id: schema.sql 972 2012-01-29 11:00:09Z matthew $
--

-- Copyright (c) 2012 Matthew Seaman. All rights reserved.
--
-- Redistribution and use in source and binary forms, with or without
-- modification, are permitted provided that the following conditions
-- are met:
--
--    1.  Redistributions of source code must retain the above
--        copyright notice, this list of conditions and the following
--        disclaimer.
--
--    2.  Redistributions in binary form must reproduce the above
--        copyright notice, this list of conditions and the following
--        disclaimer in the documentation and/or other materials
--        provided with the distribution.
--
-- THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS''
-- AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
-- TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-- PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR
-- CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-- SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-- LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
-- USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
-- ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
-- OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
-- OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
-- SUCH DAMAGE.

COMMENT ON DATABASE portindex IS 'Ports Indexing';

-- Create the schema_version table 

CREATE TABLE schema_version (
    version text NOT NULL
);

INSERT INTO
    schema_version(version)
VALUES
    (split_part('$Revision: 972 $', ' ', 2))
;

-- Create the makefile_variables type and the raw_data table

CREATE TYPE makefile_variables_type AS ENUM (
    '_is_a',
    '_makefile_mtime',
    '_sha256',
    '_size',
    '.CURDIR',
    '.MAKEFILE_LIST',
    'BROKEN',
    'BUILD_DEPENDS',
    'CATEGORIES',
    'COMMENT',
    'DEPRECATED',
    'DESCR',
    'DISTINFO_FILE',
    'EXPIRATION_DATE',
    'EXTRACT_DEPENDS',
    'FETCH_DEPENDS',
    'FORBIDDEN',
    'GROUPS',
    'LATEST_LINK',
    'LIB_DEPENDS',
    'LICENSE',
    'MAINTAINER',
    'MASTER_PORT',
    'NO_CDROM',
    'NO_LATEST_LINK',
    'NO_PACKAGE',
    'NOT_FOR_ARCHS',
    'ONLY_FOR_ARCHS',
    'OPTIONS',
    'OPTIONSFILE',
    'PATCH_DEPENDS',
    'PKGNAME',
    'PREFIX',
    'RESTRICTED',
    'RUN_DEPENDS',
    'SUBDIR',
    'TEST_DEPENDS',
    'USERS',
    'WWW'
);

CREATE TABLE raw_data (
    name text NOT NULL,
    parent text NOT NULL,
    last_modified timestamp with time zone DEFAULT now(),
    variable_name makefile_variables_type NOT NULL,
    data text NOT NULL,
    UNIQUE(name, parent, variable_name, data)
);

-- Create the port_object type and the main ports_tree table

CREATE TYPE port_object AS ENUM (
    'root',
    'category',
    'port',
    'virtual-category'
);

CREATE TABLE ports_tree (
    port_id serial PRIMARY KEY,
    name text NOT NULL,
    type port_object NOT NULL,
    parent_id integer REFERENCES ports_tree(port_id),
    UNIQUE(name, parent_id)
);

CREATE VIEW ports_by_name AS
    SELECT
        port.port_id, port.name, parent.name
    AS
        parent, port.type
    FROM (
        ports_tree port LEFT JOIN ports_tree parent 
            ON (port.parent_id = parent.port_id)
    )
;

-- Create the makefile and makefile_dependency tables

CREATE TABLE makefile (
    makefile_id serial PRIMARY KEY,
    path text NOT NULL,
    mtime timestamp with time zone,
    UNIQUE(path)
);

CREATE TABLE makefile_dependency (
    makefile_id integer NOT NULL REFERENCES makefile(makefile_id),
    include_id integer NOT NULL REFERENCES makefile(makefile_id),
    port_id integer NOT NULL REFERENCES ports_tree(port_id),
    UNIQUE(makefile_id, include_id, port_id)
);

-- Create the maintainer table

CREATE TABLE maintainer (
    maintainer_id serial PRIMARY KEY,
    maintainer_email text NOT NULL,
    UNIQUE(maintainer_email)
);

-- Create the port table

CREATE TABLE port (
    port_id integer NOT NULL REFERENCES ports_tree(port_id),
    maintainer_id integer NOT NULL REFERENCES maintainer(maintainer_id),
    comment text NOT NULL,
    descr text NOT NULL,
    pkg_name text NOT NULL,
    prefix text NOT NULL,
    www text,
    UNIQUE(port_id)
);

-- Create the latest_link_all table and the latest_link view

CREATE TABLE latest_link_all (
    port_id integer NOT NULL REFERENCES ports_tree(port_id),
    latest_link text NOT NULL,
    no_latest_link boolean NOT NULL DEFAULT false,
    UNIQUE(port_id)
);

CREATE VIEW latest_link AS 
    SELECT
        port_id, latest_link
    FROM
        latest_link_all
    WHERE
        NOT no_latest_link
;

-- Create the category table

CREATE TABLE category (
    port_id integer NOT NULL REFERENCES ports_tree(port_id),
    category_id integer NOT NULL REFERENCES ports_tree(port_id),
    ordering integer NOT NULL,
    UNIQUE(port_id, category_id, ordering)
);

-- Create the foo_depends and foo_depends_accumulated tables

-- FETCH_DEPENDS

CREATE TABLE fetch_depends (
    port_id integer NOT NULL REFERENCES ports_tree(port_id),
    dependency_id integer NOT NULL REFERENCES ports_tree(port_id),
    UNIQUE(port_id, dependency_id)
);

CREATE TABLE fetch_depends_accumulated (
    port_id integer NOT NULL REFERENCES ports_tree(port_id),
    dependency_id integer NOT NULL REFERENCES ports_tree(port_id),
    UNIQUE(port_id, dependency_id)
);

-- EXTRACT_DEPENDS

CREATE TABLE extract_depends (
    port_id integer NOT NULL REFERENCES ports_tree(port_id),
    dependency_id integer NOT NULL REFERENCES ports_tree(port_id),
    UNIQUE(port_id, dependency_id)
);

CREATE TABLE extract_depends_accumulated (
    port_id integer NOT NULL REFERENCES ports_tree(port_id),
    dependency_id integer NOT NULL REFERENCES ports_tree(port_id),
    UNIQUE(port_id, dependency_id)
);

-- PATCH_DEPENDS

CREATE TABLE patch_depends (
    port_id integer NOT NULL REFERENCES ports_tree(port_id),
    dependency_id integer NOT NULL REFERENCES ports_tree(port_id),
    UNIQUE(port_id, dependency_id)
);

CREATE TABLE patch_depends_accumulated (
    port_id integer NOT NULL REFERENCES ports_tree(port_id),
    dependency_id integer NOT NULL REFERENCES ports_tree(port_id),
    UNIQUE(port_id, dependency_id)
);

-- BUILD_DEPENDS (+LIB for accumulation)

CREATE TABLE build_depends (
    port_id integer NOT NULL REFERENCES ports_tree(port_id),
    dependency_id integer NOT NULL REFERENCES ports_tree(port_id),
    UNIQUE(port_id, dependency_id)
);

CREATE TABLE build_lib_depends (
    port_id integer NOT NULL REFERENCES ports_tree(port_id),
    dependency_id integer NOT NULL REFERENCES ports_tree(port_id),
    UNIQUE(port_id, dependency_id)
);

CREATE TABLE build_depends_accumulated (
    port_id integer NOT NULL REFERENCES ports_tree(port_id),
    dependency_id integer NOT NULL REFERENCES ports_tree(port_id),
    UNIQUE(port_id, dependency_id)
);

-- RUN_DEPENDS (+LIB for accumuation)

CREATE TABLE run_depends (
    port_id integer NOT NULL REFERENCES ports_tree(port_id),
    dependency_id integer NOT NULL REFERENCES ports_tree(port_id),
    UNIQUE(port_id, dependency_id)
);

CREATE TABLE run_lib_depends (
    port_id integer NOT NULL REFERENCES ports_tree(port_id),
    dependency_id integer NOT NULL REFERENCES ports_tree(port_id),
    UNIQUE(port_id, dependency_id)
);

CREATE TABLE run_depends_accumulated (
    port_id integer NOT NULL REFERENCES ports_tree(port_id),
    dependency_id integer NOT NULL REFERENCES ports_tree(port_id),
    UNIQUE(port_id, dependency_id)
);

-- LIB_DEPENDS

CREATE TABLE lib_depends (
    port_id integer NOT NULL REFERENCES ports_tree(port_id),
    dependency_id integer NOT NULL REFERENCES ports_tree(port_id),
    UNIQUE(port_id, dependency_id)
);

-- TEST_DEPENDS

CREATE TABLE test_depends (
    port_id integer NOT NULL REFERENCES ports_tree(port_id),
    dependency_id integer NOT NULL REFERENCES ports_tree(port_id),
    UNIQUE(port_id, dependency_id)
);

CREATE TABLE test_depends_accumulated (
    port_id integer NOT NULL REFERENCES ports_tree(port_id),
    dependency_id integer NOT NULL REFERENCES ports_tree(port_id),
    UNIQUE(port_id, dependency_id)
);

-- Create the master_slave table

CREATE TABLE master_slave (
    master_id integer NOT NULL REFERENCES ports_tree(port_id),
    slave_id integer NOT NULL REFERENCES ports_tree(port_id),
    UNIQUE(master_id, slave_id)
);

-- Create the license and license_dependency tables

CREATE TABLE license (
    license_id serial PRIMARY KEY,
    license_name text NOT NULL,
    UNIQUE(license_name)
);

CREATE TABLE license_dependency (
    license_id integer NOT NULL REFERENCES license(license_id),
    port_id integer NOT NULL REFERENCES ports_tree(port_id),
    UNIQUE(license_id, port_id)
);

-- Create the user_group table and associated type

CREATE TYPE user_group_type AS ENUM(
    'user',
    'group'
);

CREATE TABLE user_group (
    user_group_id serial PRIMARY KEY,
    user_group_type user_group_type NOT NULL,
    user_group_name text NOT NULL,
    UNIQUE(user_group_type, user_group_name)
);

CREATE TABLE user_group_dependency (
    port_id integer REFERENCES ports_tree(port_id) NOT NULL,
    user_group_id integer REFERENCES user_group(user_group_id) NOT NULL,
    UNIQUE(port_id, user_group_id)
);

-- Create the caveats table and associated type

CREATE TYPE caveat_type AS ENUM (
    'BROKEN',
    'DEPRECATED',
    'EXPIRATION_DATE',
    'FORBIDDEN', 
    'NO_CDROM',
    'NO_PACKAGE',
    'NOT_FOR_ARCHS',
    'ONLY_FOR_ARCHS',
    'RESTRICTED'
);

CREATE TABLE caveat (
    port_id integer REFERENCES ports_tree(port_id) NOT NULL,
    caveat_type caveat_type NOT NULL,
    caveat text NOT NULL,
    UNIQUE(port_id, caveat_type, caveat)
);

-- Create distfile and distfile_dependency table

CREATE TABLE distfile (
    distfile_id serial PRIMARY KEY,
    distfile_path text NOT NULL,
    distfile_size integer NOT NULL,
    distfile_sha256 text NOT NULL,
    UNIQUE(distfile_path, distfile_size, distfile_sha256)
);

CREATE TABLE distfile_dependency (
    port_id integer REFERENCES ports_tree(port_id),
    distfile_id integer REFERENCES distfile(distfile_id),
    UNIQUE(port_id, distfile_id)
);

-- Create option table

CREATE TABLE option (
    port_id integer REFERENCES ports_tree(port_id) NOT NULL,
    option_name text NOT NULL,
    option_text text NOT NULL,
    enabled boolean NOT NULL,
    UNIQUE(port_id, option_name)
);

-- Create optionsfile tables

CREATE TABLE optionsfile (
    optionsfile_id serial PRIMARY KEY,
    optionsfile text NOT NULL,
    UNIQUE(optionsfile)
);

CREATE TABLE optionsfile_dependency (
    optionsfile_id integer NOT NULL REFERENCES optionsfile(optionsfile_id),
    port_id integer NOT NULL REFERENCES ports_tree(port_id),
    UNIQUE(optionsfile_id, port_id)
);

--
-- That's All Folks!
--
